"use strict";
this.name = "oolite-random-hits-hunter";
this.author = "LittleBear, Eric Walch & Thargoid";
this.copyright = "November 2008 - But do what you like with it in your OXPs!";
this.description	= "Gives the Bounty Hunter NPCs from the Space Bars an individual name and rank on the ID computer and sets them to the right AI. Also causes hunters to regard hits from the spacebar's turret gunners or other hunters as friendly fire and ignore rather than attack."
this.licence     = "CC BY-NC-SA 3.0";
this.version = "2.4";
 
/*
Oolites random generator has a flaw that it is regularly reset by ships asking for the witchspace location. 
This results in that "random" descriptions often give the same result. This problem is not present on system
setup, but after ships starting to fly. And because this ship is added afterwards by launching from a
seedy bar, they tend to get the same name. I tryed to fix this by first doing a random number of string
expansions. Its easy fixable in Oolite itself but other oxp's might want the predictable "randomness"
that is different for different systems.
*/

this.shipSpawned = function ()
{
    //this.ship.name = "Random Hits Patrol";
    var count = Math.round(Math.random()*15);
	var level = parseInt(this.ship.scriptInfo.level);
	var name = "";
	if (0 < oolite.compareVersion("1.75.4"))
	{
		// Before 1.75.4 radomness of string expantions at this point was bad.
		for (var i=0; i<count; i++) name = expandDescription("[nom11]"); // mix up the random generator.
	}
	switch(level)
	{
		case 1: name = expandDescription("[random_hits_hunterlow_name]"); break;
		case 2: name = expandDescription("[random_hits_huntermedium_name]"); break;
		case 3: name = expandDescription("[random_hits_hunterhigh_name]"); break;
		default: name = "unknown"; // means a wrong level definition in shipData.plist. 
	}
	this.ship.displayName = this.ship.name+": "+name; // Show the Hunters name on the ID Computer when targeted.
	
    this.legalPenalty = 4*count + 25;
    delete this.shipSpawned;
}

this.shipBeingAttacked = function(who)
{
    if (!who) return;
    this.ship.bounty = 0;
    if (who.primaryRole === "random_hits_patrol") 
    {
        this.ship.reactToAIMessage("FRIENDLY_FIRE");
        if (who.target == this.ship) who.reactToAIMessage("TARGET_LOST");
        return;		
    }
	if (who.isPolice) 
    {
        this.ship.reactToAIMessage("FRIENDLY_FIRE");
        if (who.target == this.ship) who.reactToAIMessage("TARGET_LOST");
        return;		
    }
				
    if (who.primaryRole === "random_hits_spacebar")
    {
        this.ship.reactToAIMessage("FRIENDLY_FIRE");
        who.reactToAIMessage("TARGET_LOST");
        if (who.target == this.ship) who.reactToAIMessage("TARGET_LOST");
        return;		
    }
	if (who.isPlayer) 
    {
        this.ship.reactToAIMessage("ENEMY_ATTACK"); 
        player.bounty |= this.legalPenalty;
        return;		
    }				
    else
    {
        this.ship.reactToAIMessage("ENEMY_ATTACK"); // The AIs of the bars and hunters do not react to the nomal "ATTACKED" message, only this special one. They therefore ignore attacks by their own side.	
    }
}

this.shipDied = function (whom)
{
	missionVariables.random_hits_hunter_kill_score++

    if (whom && whom.isPlayer)
	{
        missionVariables.random_hits_criminal_hate_score -= parseInt(this.ship.scriptInfo.hateScore);
        player.bounty += parseInt(this.ship.scriptInfo.bounty);
	}
}

this.shipWillEnterWormhole = function ()
{
    if (this.ship.group && this.ship.group.leader && this.ship.group.leader.script.name === "oolite-random-hits-spacebar")
    {
        // increase the available patrol count of the spacebar when it is part of the spacebar group.
        this.ship.group.leader.script.patrols++;
        this.ship.group = null; // release from the group after jumping.
    }
};

this.doScan = function () 
{ 
	// create an array with all ships (exclude barrels and rocks) 
	function allShips(entity) {return entity.isShip && entity.scanClass !== "CLASS_ROCK" && entity.scanClass !== "CLASS_CARGO"}; 
	var anyShips = system.filteredEntities(this, allShips, this.ship, this.ship.scannerRange);
    
    // anyShips now holds all real ships in scanner range. From now only analyse this small list further to save time.
    
    // Unlike AI scans this scan already sets the target. so don't use "setTargetToFoundTarget" because the foundTarget
    // could be something completely unexpected that was left behind from a previous scan.

	// do player first.
	function isPlayer(entity) {return entity.isPlayer}; 
    var playerPresent = (anyShips.filter(isPlayer).length > 0);
    if (playerPresent && player.ship.bounty > 0)
    {
        this.ship.target = player.ship;
        this.ship.reactToAIMessage("OFFENDER_FOUND");
        return;
    }

	// do fugitive ships (Most thargoids are also fugitive, so noneed for a special scan)
	function fugitiveShips(entity) {return entity.bounty > 64};
    var fugitives = anyShips.filter(fugitiveShips); 
    if (fugitives.length > 0){
        this.ship.target = fugitives[Math.floor(Math.random()*fugitives.length)];
        this.ship.reactToAIMessage("OFFENDER_FOUND");
        return;
    } 
    
    // do offender ships, forget about minor offenders.
	function offenderShips(entity) {return entity.bounty > 5}; 
    var offenders = anyShips.filter(offenderShips); 
    if (offenders.length > 0){
        this.ship.target = offenders[Math.floor(Math.random()*offenders.length)];
        this.ship.reactToAIMessage("OFFENDER_FOUND");
    } else {
        this.ship.reactToAIMessage("NOTHING_FOUND");
    }
};

/*
Get exit coordinates at a random location on a sphere round the station. This is called 3 times after a fight. That way the
ship makes 3 random patrol legs around the station before jumpung out. It is scanning during these patrols, but probably
the scanning by the hunters can be omitted as the station also scanns and issues a groupAttackShip to its
hunters when it finds a target. By also letting the hunters scan, they will be able to find ships outside the bar's scan range.
*/
this.getPatrolCoordinates = function ()
{
    var pos;
    if (this.ship.group && this.ship.group.leader) 
    {
        pos = this.ship.group.leader.position; // bar position
    }
    else
    {
        pos = this.ship.position; // happens only when the bar has exploded
    }
    
    pos = pos.add(Vector3D.randomDirection(20E3));
    this.ship.savedCoordinates = pos;
};

this.getExitCoordinates = function ()
{
    var pos;
    if (this.ship.group && this.ship.group.leader) 
    {
        pos = this.ship.group.leader.position; // bar position
        var direction = this.ship.position.subtract(pos).direction(); // unit vector pointing from bar to ship.
        pos = pos.add(direction.multiply(35E3));
    }
    else
    {
        pos = this.ship.position; // happens only when the bar has exploded
    }
    
    this.ship.savedCoordinates = pos;
}
